@@ -6,6 +6,7 @@ import java.math.BigInteger;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Map;
+import java.util.regex.Pattern;
 
 import org.yaml.snakeyaml.DumperOptions;
 import org.yaml.snakeyaml.DumperOptions.FlowStyle;
@@ -27,7 +28,7 @@ public class YAMLGenerator extends GeneratorBase
         /**
          * Whether we are to write an explicit document start marker ("---")
          * or not.
-         * 
+         *
          * @since 2.3
          */
         WRITE_DOC_START_MARKER(true),
@@ -37,17 +38,17 @@ public class YAMLGenerator extends GeneratorBase
          * or "generic" Object Id mechanism (false). Former works better for systems that
          * are YAML-centric; latter may be better choice for interoperability, when
          * converting between formats or accepting other formats.
-         * 
+         *
          * @since 2.5
          */
         USE_NATIVE_OBJECT_ID(true),
-        
+
         /**
          * Whether to use YAML native Type Id construct for indicating type (true);
          * or "generic" type property (false). Former works better for systems that
          * are YAML-centric; latter may be better choice for interoperability, when
          * converting between formats or accepting other formats.
-         * 
+         *
          * @since 2.5
          */
         USE_NATIVE_TYPE_ID(true),
@@ -78,13 +79,24 @@ public class YAMLGenerator extends GeneratorBase
          *
          * @since 2.7
          */
-        MINIMIZE_QUOTES(false)
-        
+        MINIMIZE_QUOTES(false),
+
+        /**
+         * Whether numbers stored as strings will be rendered with quotes (true) or
+         * without quotes (false, default) when MINIMIZE_QUOTES is enabled.
+         *<p>
+         * Minimized quote usage makes for more human readable output; however, content is
+         * limited to printable characters according to the rules of
+         * <a href="http://www.yaml.org/spec/1.2/spec.html#style/block/literal">literal block style</a>.
+         *
+         * @since 2.8.2
+         */
+        ALWAYS_QUOTE_NUMBERS_AS_STRINGS(false)
         ;
 
         protected final boolean _defaultState;
         protected final int _mask;
-        
+
         /**
          * Method that calculates bit set (flags) of all features that
          * are enabled by default.
@@ -99,20 +111,27 @@ public class YAMLGenerator extends GeneratorBase
             }
             return flags;
         }
-        
+
         private Feature(boolean defaultState) {
             _defaultState = defaultState;
             _mask = (1 << ordinal());
         }
-        
+
         public boolean enabledByDefault() { return _defaultState; }
-        public boolean enabledIn(int flags) { return (flags & _mask) != 0; }        
+        public boolean enabledIn(int flags) { return (flags & _mask) != 0; }
         public int getMask() { return _mask; }
-    };
+    }
+
+    /*
+    /**********************************************************
+    /* Internal constants
+    /**********************************************************
+     */
 
     protected final static long MIN_INT_AS_LONG = (long) Integer.MIN_VALUE;
     protected final static long MAX_INT_AS_LONG = (long) Integer.MAX_VALUE;
-    
+    protected final static Pattern PLAIN_NUMBER_P = Pattern.compile("[0-9]*(\\.[0-9]*)?");
+
     /*
     /**********************************************************
     /* Configuration
@@ -134,7 +153,7 @@ public class YAMLGenerator extends GeneratorBase
 
     // for field names, leave out quotes
     private final static Character STYLE_NAME = null;
-    
+
     // numbers, booleans, should use implicit
     private final static Character STYLE_SCALAR = null;
     // Strings quoted for fun
@@ -166,7 +185,7 @@ public class YAMLGenerator extends GeneratorBase
      * need to output one.
      */
     protected String _typeId;
-    
+
     /*
     /**********************************************************
     /* Life-cycle
@@ -184,14 +203,14 @@ public class YAMLGenerator extends GeneratorBase
         _writer = out;
 
         _outputOptions = buildDumperOptions(jsonFeatures, yamlFeatures, version);
-        
+
         _emitter = new Emitter(_writer, _outputOptions);
         // should we start output now, or try to defer?
         _emitter.emit(new StreamStartEvent(null, null));
         Map<String,String> noTags = Collections.emptyMap();
-        
+
         boolean startMarker = Feature.WRITE_DOC_START_MARKER.enabledIn(yamlFeatures);
-        
+
         _emitter.emit(new DocumentStartEvent(null, null, startMarker,
                 version, // for 1.10 was: ((version == null) ? null : version.getArray()),
                 noTags));
@@ -213,10 +232,10 @@ public class YAMLGenerator extends GeneratorBase
         return opt;
     }
 
-    /*                                                                                       
-    /**********************************************************                              
-    /* Versioned                                                                             
-    /**********************************************************                              
+    /*
+    /**********************************************************
+    /* Versioned
+    /**********************************************************
      */
 
     @Override
@@ -272,12 +291,15 @@ public class YAMLGenerator extends GeneratorBase
         _formatFeatures = (_formatFeatures & ~mask) | (values & mask);
         return this;
     }
-    
+
     @Override
     public boolean canUseSchema(FormatSchema schema) {
         return false;
     }
-    
+
+    @Override
+    public boolean canWriteFormattedNumbers() { return true; }
+
     //@Override public void setSchema(FormatSchema schema)
 
     /*
@@ -308,13 +330,13 @@ public class YAMLGenerator extends GeneratorBase
         }
         return this;
     }
-    
+
     /*
     /**********************************************************************
     /* Overridden methods; writing field names
     /**********************************************************************
      */
-    
+
     /* And then methods overridden to make final, streamline some
      * aspects...
      */
@@ -367,7 +389,7 @@ public class YAMLGenerator extends GeneratorBase
     {
         _writer.flush();
     }
-    
+
     @Override
     public void close() throws IOException
     {
@@ -384,7 +406,7 @@ public class YAMLGenerator extends GeneratorBase
     /* Public API: structural output
     /**********************************************************
      */
-    
+
     @Override
     public final void writeStartArray() throws IOException
     {
@@ -400,15 +422,15 @@ public class YAMLGenerator extends GeneratorBase
         _emitter.emit(new SequenceStartEvent(anchor, yamlTag,
                 implicit,  null, null, style));
     }
-    
+
     @Override
     public final void writeEndArray() throws IOException
     {
         if (!_writeContext.inArray()) {
-            _reportError("Current context not an ARRAY but "+_writeContext.getTypeDesc());
+            _reportError("Current context not Array but "+_writeContext.typeDesc());
         }
         // just to make sure we don't "leak" type ids
-        _typeId = null;        
+        _typeId = null;
         _writeContext = _writeContext.getParent();
         _emitter.emit(new SequenceEndEvent(null, null));
     }
@@ -433,10 +455,10 @@ public class YAMLGenerator extends GeneratorBase
     public final void writeEndObject() throws IOException
     {
         if (!_writeContext.inObject()) {
-            _reportError("Current context not an object but "+_writeContext.getTypeDesc());
+            _reportError("Current context not Object but "+_writeContext.typeDesc());
         }
         // just to make sure we don't "leak" type ids
-        _typeId = null;        
+        _typeId = null;
         _writeContext = _writeContext.getParent();
         _emitter.emit(new MappingEndEvent(null, null));
     }
@@ -456,17 +478,24 @@ public class YAMLGenerator extends GeneratorBase
         }
         _verifyValueWrite("write String value");
         Character style = STYLE_QUOTED;
-        if (Feature.MINIMIZE_QUOTES.enabledIn(_formatFeatures)) {
-            if (text.contains("\n")) {
+        if (Feature.MINIMIZE_QUOTES.enabledIn(_formatFeatures) && !isBooleanContent(text)) {
+          // If this string could be interpreted as a number, it must be quoted.
+            if (Feature.ALWAYS_QUOTE_NUMBERS_AS_STRINGS.enabledIn(_formatFeatures)
+                    && PLAIN_NUMBER_P.matcher(text).matches()) {
+                style = STYLE_QUOTED;
+            } else if (text.indexOf('\n') >= 0) {
                 style = STYLE_LITERAL;
-            }
-            else {
+            } else {
                 style = STYLE_PLAIN;
             }
         }
         _writeScalar(text, "string", style);
     }
 
+    private boolean isBooleanContent(String text) {
+        return text.equals("true") || text.equals("false");
+    }
+
     @Override
     public void writeString(char[] text, int offset, int len) throws IOException
     {
@@ -540,7 +569,7 @@ public class YAMLGenerator extends GeneratorBase
     /* Output method implementations, base64-encoded binary
     /**********************************************************
      */
-    
+
     @Override
     public void writeBinary(Base64Variant b64variant, byte[] data, int offset, int len) throws IOException
     {
@@ -599,13 +628,13 @@ public class YAMLGenerator extends GeneratorBase
         _verifyValueWrite("write number");
         _writeScalar(String.valueOf(v.toString()), "java.math.BigInteger", STYLE_SCALAR);
     }
-    
+
     @Override
     public void writeNumber(double d) throws IOException
     {
         _verifyValueWrite("write number");
         _writeScalar(String.valueOf(d), "double", STYLE_SCALAR);
-    }    
+    }
 
     @Override
     public void writeNumber(float f) throws IOException
@@ -656,14 +685,14 @@ public class YAMLGenerator extends GeneratorBase
         // yes, YAML does support Native Type Ids!
         // 10-Sep-2014, tatu: Except as per [#23] might not want to...
         return Feature.USE_NATIVE_OBJECT_ID.enabledIn(_formatFeatures);
-    }    
+    }
 
     @Override
     public boolean canWriteTypeId() {
         // yes, YAML does support Native Type Ids!
         // 10-Sep-2014, tatu: Except as per [#22] might not want to...
         return Feature.USE_NATIVE_TYPE_ID.enabledIn(_formatFeatures);
-    }    
+    }
 
     @Override
     public void writeTypeId(Object id)
@@ -681,7 +710,7 @@ public class YAMLGenerator extends GeneratorBase
         AliasEvent evt = new AliasEvent(String.valueOf(id), null, null);
         _emitter.emit(evt);
     }
-    
+
     @Override
     public void writeObjectId(Object id)
         throws IOException
@@ -724,7 +753,7 @@ public class YAMLGenerator extends GeneratorBase
     {
         _emitter.emit(_scalarEvent(value, style));
     }
-    
+
     protected ScalarEvent _scalarEvent(String value, Character style)
     {
         String yamlTag = _typeId;
